@@ -295,8 +295,9 @@ get_global_customization_uri (const char *customization_name)
 {
 	char *directory_path, *directory_uri;
 	
-	directory_path = nautilus_make_path (NAUTILUS_DATADIR,
-					     customization_name);
+	directory_path = g_build_filename (NAUTILUS_DATADIR,
+					   customization_name,
+					   NULL);
 	directory_uri = gnome_vfs_get_uri_from_local_path (directory_path);
 	
 	g_free (directory_path);
@@ -320,8 +321,9 @@ get_private_customization_uri (const char *customization_name)
 	char *directory_path, *directory_uri;
 
 	user_directory = nautilus_get_user_directory ();
-	directory_path = nautilus_make_path (user_directory,
-					     customization_name);
+	directory_path = g_build_filename (user_directory,
+					   customization_name,
+					   NULL);
 	g_free (user_directory);
 	directory_uri = gnome_vfs_get_uri_from_local_path (directory_path);
 	g_free (directory_path);
@@ -342,7 +344,7 @@ get_file_path_for_mode (const NautilusCustomizationData *data,
 		directory_uri = get_private_customization_uri (data->customization_name);
 	}
 	
-	uri = nautilus_make_path (directory_uri, file_name);
+	uri = g_build_filename (directory_uri, file_name, NULL);
 	g_free (directory_uri);
 	directory_name = gnome_vfs_get_local_path_from_uri (uri);
 	g_free (uri);
@@ -394,22 +396,29 @@ nautilus_customization_make_pattern_chit (GdkPixbuf *pattern_tile, GdkPixbuf *fr
 {
 	GdkPixbuf *pixbuf, *temp_pixbuf;
 	int frame_width, frame_height;
+	int pattern_width, pattern_height;
 	
 	
 	frame_width = gdk_pixbuf_get_width (frame);
 	frame_height = gdk_pixbuf_get_height (frame);
-	
+	pattern_width = gdk_pixbuf_get_width (pattern_tile);
+	pattern_height = gdk_pixbuf_get_height (pattern_tile);
+
+	pixbuf = gdk_pixbuf_copy (frame);
+
 	/* scale the pattern tile to the proper size */
-	pixbuf = gdk_pixbuf_scale_simple (pattern_tile, frame_width, frame_height, GDK_INTERP_BILINEAR);
-			
-	/* composite the mask on top of it */
-	gdk_pixbuf_composite (frame, pixbuf, 0, 0, frame_width, frame_height,
-			      0.0, 0.0, 1.0, 1.0, GDK_INTERP_BILINEAR, 255);
+	gdk_pixbuf_scale (pattern_tile,
+			  pixbuf,
+			  2, 2, frame_width - 8, frame_height - 8,
+			  0, 0,
+			  (double)(frame_width - 8 + 1)/pattern_width,
+			  (double)(frame_height - 8 + 1)/pattern_height,
+			  GDK_INTERP_BILINEAR);
 	
 	/* if we're dragging, get rid of the light-colored halo */
 	if (dragging) {
-		temp_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, frame_width - 6, frame_height - 6);
-		gdk_pixbuf_copy_area (pixbuf, 2, 2, frame_width - 6, frame_height - 6, temp_pixbuf, 0, 0);
+		temp_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, frame_width - 8, frame_height - 8);
+		gdk_pixbuf_copy_area (pixbuf, 2, 2, frame_width - 8, frame_height - 8, temp_pixbuf, 0, 0);
 		g_object_unref (pixbuf);
 		pixbuf = temp_pixbuf;
 	}
@@ -468,7 +477,7 @@ load_name_map_hash_table (NautilusCustomizationData *data)
 	data->name_map_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	
 	/* build the path name to the browser.xml file and load it */
-	xml_path = nautilus_make_path (NAUTILUS_DATADIR, "browser.xml");
+	xml_path = g_build_filename (NAUTILUS_DATADIR, "browser.xml", NULL);
 	if (xml_path) {
 		browser_data = xmlParseFile (xml_path);
 		g_free (xml_path);
