@@ -117,10 +117,12 @@ typedef struct {
 	int last_icon_position_index;
 	GList *uris;
 	const GList *last_uri;
+	int screen;
 } IconPositionIterator;
 
 static IconPositionIterator *
-icon_position_iterator_new (GArray *icon_positions, const GList *uris)
+icon_position_iterator_new (GArray *icon_positions, const GList *uris,
+			    int screen)
 {
 	IconPositionIterator *result;
 	guint index;
@@ -137,6 +139,7 @@ icon_position_iterator_new (GArray *icon_positions, const GList *uris)
 
 	result->uris = eel_g_str_list_copy ((GList *)uris);
 	result->last_uri = result->uris;
+	result->screen = screen;
 
 	return result;
 }
@@ -497,6 +500,7 @@ typedef enum {
 	ERROR_NOT_WRITABLE,
 	ERROR_NOT_ENOUGH_PERMISSIONS,
 	ERROR_NO_SPACE,
+	ERROR_SOURCE_IN_TARGET,
 	ERROR_OTHER
 } NautilusFileOperationsErrorKind;
 
@@ -581,10 +585,19 @@ build_error_string (const char *source_name, const char *target_name,
 		 */
 		switch (operation_kind) {
 		case TRANSFER_MOVE:
-			if (error_kind == ERROR_NOT_ENOUGH_PERMISSIONS) {
+			switch (error_kind) {
+			case ERROR_NOT_ENOUGH_PERMISSIONS:
 				error_string = _("Error while moving.\n\n"
 						 "\"%s\" cannot be moved because you do not have "
 						 "permissions to change it or its parent folder.");
+				break;
+			case ERROR_SOURCE_IN_TARGET:
+				error_string = _("Error while moving.\n\n"
+						 "\"%s\" cannot be moved because it or its parent folder "
+						 "are contained in the destination.");
+				break;
+			default:
+				break;
 			}
 			break;
 		case TRANSFER_MOVE_TO_TRASH:
@@ -862,6 +875,10 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 		} else if (progress_info->vfs_status == GNOME_VFS_ERROR_NO_SPACE) {
 			error_location = ERROR_LOCATION_TARGET;
 			error_kind = ERROR_NO_SPACE;
+		} else if (progress_info->vfs_status == GNOME_VFS_ERROR_DIRECTORY_NOT_EMPTY
+			   && transfer_info->kind == TRANSFER_MOVE) {
+			error_location = ERROR_LOCATION_SOURCE_OR_PARENT;
+			error_kind = ERROR_SOURCE_IN_TARGET;
 		}
 
 		text = build_error_string (formatted_source_name, formatted_target_name,
@@ -890,7 +907,8 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 			break;
 		}
 
-		if (error_location == ERROR_LOCATION_TARGET) {
+		if (error_location == ERROR_LOCATION_TARGET ||
+		    error_kind == ERROR_SOURCE_IN_TARGET) {
 			/* We can't continue, just tell the user. */
 			eel_run_simple_dialog (parent_for_error_dialog (transfer_info),
 				TRUE, text, dialog_title, _("Stop"), NULL);
@@ -1510,7 +1528,7 @@ apply_one_position (IconPositionIterator *position_iterator,
 	GdkPoint point;
 
 	if (icon_position_iterator_get_next (position_iterator, source_name, &point)) {
-		nautilus_file_changes_queue_schedule_position_set (target_name, point);
+		nautilus_file_changes_queue_schedule_position_set (target_name, point, position_iterator->screen);
 	} else {
 		nautilus_file_changes_queue_schedule_position_remove (target_name);
 	}
@@ -1700,6 +1718,9 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	
 	IconPositionIterator *icon_position_iterator;
 
+	GdkScreen *screen;
+	int screen_num;
+
 	g_assert (item_uris != NULL);
 
 	target_dir_uri = NULL;
@@ -1819,11 +1840,13 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	/* set up the copy/move parameters */
 	transfer_info = transfer_info_new (parent_view);
 	if (relative_item_points != NULL && relative_item_points->len > 0) {
+		screen = gtk_widget_get_screen (GTK_WIDGET (parent_view));
+		screen_num = gdk_screen_get_number (screen);
 		/* FIXME: we probably don't need an icon_position_iterator
 		 * here at all.
 		 */
 		icon_position_iterator = icon_position_iterator_new
-			(relative_item_points, item_uris);
+			(relative_item_points, item_uris, screen_num);
 	} else {
 		icon_position_iterator = NULL;
 	}
