@@ -42,16 +42,15 @@ GNOME_CLASS_BOILERPLATE (FMIconContainer, fm_icon_container,
 static FMIconView *
 get_icon_view (NautilusIconContainer *container)
 {
-	g_assert (FM_IS_ICON_CONTAINER (container));
-
-	return FM_ICON_CONTAINER (container)->view;
+	/* Type unsafe comparison for performance */
+	return ((FMIconContainer *)container)->view;
 }
 
-static NautilusScalableIcon *
+static char *
 fm_icon_container_get_icon_images (NautilusIconContainer *container,
 				   NautilusIconData      *data,
-				   const char            *modifier,
-				   GList                **emblem_icons)
+				   GList                **emblem_icons,
+				   char                 **embedded_text)
 {
 	FMIconView *icon_view;
 	EelStringList *emblems_to_ignore;
@@ -63,6 +62,10 @@ fm_icon_container_get_icon_images (NautilusIconContainer *container,
 	icon_view = get_icon_view (container);
 	g_return_val_if_fail (icon_view != NULL, NULL);
 
+	if (embedded_text) {
+		*embedded_text = nautilus_file_peek_top_left_text (file);
+	}
+	
 	if (emblem_icons != NULL) {
 		emblems_to_ignore = fm_directory_view_get_emblem_names_to_exclude 
 			(FM_DIRECTORY_VIEW (icon_view));
@@ -71,7 +74,7 @@ fm_icon_container_get_icon_images (NautilusIconContainer *container,
 		eel_string_list_free (emblems_to_ignore);
 	}
 
-	return nautilus_icon_factory_get_icon_for_file (file, modifier);
+	return nautilus_icon_factory_get_icon_for_file (file);
 }
 
 /*
@@ -170,7 +173,7 @@ fm_icon_container_get_icon_text (NautilusIconContainer *container,
 				 char                 **editable_text,
 				 char                 **additional_text)
 {
-	char *actual_uri, *path;
+	char *actual_uri;
 	gchar *description;
 	char *attribute_names;
 	char **text_array;
@@ -198,16 +201,14 @@ fm_icon_container_get_icon_text (NautilusIconContainer *container,
 	/* Handle link files specially. */
 	if (nautilus_file_is_nautilus_link (file)) {
 		/* FIXME bugzilla.gnome.org 42531: Does sync. I/O and works only locally. */
-		actual_uri = nautilus_file_get_uri (file);
-		path = gnome_vfs_get_local_path_from_uri (actual_uri);
-		g_free (actual_uri);
  		*additional_text = NULL;
-		if (path != NULL) {
-			description = nautilus_link_local_get_additional_text (path);
-			g_free (path);
+		if (nautilus_file_is_local (file)) {
+			actual_uri = nautilus_file_get_uri (file);
+			description = nautilus_link_local_get_additional_text (actual_uri);
 			if (description)
 				*additional_text = g_strdup_printf (" \n%s\n ", description);
 			g_free (description);
+			g_free (actual_uri);
 		}
 		/* Don't show the normal extra information for desktop files, it doesn't
 		 * make sense. */
@@ -251,17 +252,6 @@ fm_icon_container_get_icon_text (NautilusIconContainer *container,
 	g_strfreev (text_array);
 }
 
-static char *
-get_local_path (NautilusFile *file)
-{
-	char *uri, *local_path;
-
-	uri = nautilus_file_get_uri (file);
-	local_path = gnome_vfs_get_local_path_from_uri (uri);
-	g_free (uri);
-	return local_path;
-}
-
 /* Sort as follows:
  *   1) home link
  *   2) mount links
@@ -278,16 +268,18 @@ typedef enum {
 static SortCategory
 get_sort_category (NautilusFile *file)
 {
-	char *path;
+	char *uri;
 	SortCategory category;
 
 	if (!nautilus_file_is_nautilus_link (file)) {
 		category = SORT_OTHER;
 	} else {
-		path = get_local_path (file);
-		g_return_val_if_fail (path != NULL, SORT_OTHER);
+		if (!nautilus_file_is_local (file))
+			return SORT_OTHER;
+
+		uri = nautilus_file_get_uri (file);
 		
-		switch (nautilus_link_local_get_link_type (path)) {
+		switch (nautilus_link_local_get_link_type (uri, NULL)) {
 		case NAUTILUS_LINK_HOME:
 			category = SORT_HOME_LINK;
 			break;
@@ -302,7 +294,7 @@ get_sort_category (NautilusFile *file)
 			break;
 		}
 		
-		g_free (path);
+		g_free (uri);
 	}
 	
 	return category;
@@ -356,9 +348,10 @@ fm_icon_container_compare_icons (NautilusIconContainer *container,
 			(container, icon_a, icon_b);
 	}
 
+	/* Type unsafe comparisons for performance */
 	return fm_icon_view_compare_files (icon_view,
-					   NAUTILUS_FILE (icon_a),
-					   NAUTILUS_FILE (icon_b));
+					   (NautilusFile *)icon_a,
+					   (NautilusFile *)icon_b);
 }
 
 static int
